# This MUST be called prior to importing any of the invoke commands
# It fixes the issue outlined here: https://github.com/pyinvoke/invoke/issues/357
from inspect import ArgSpec, getfullargspec
from typing import Any, Callable
from unittest.mock import patch

import invoke


def fix_annotations() -> None:
    """Pyinvoke doesnt accept annotations by default, this fix that.

    Based on: https://github.com/pyinvoke/invoke/pull/606
    """

    def patched_inspect_getargspec(func: Callable) -> ArgSpec:
        spec = getfullargspec(func)
        return ArgSpec(*spec[0:4])

    org_task_argspec = invoke.tasks.Task.argspec

    def patched_task_argspec(*args: Any, **kwargs: Any):
        with patch(target='inspect.getargspec', new=patched_inspect_getargspec):
            return org_task_argspec(*args, **kwargs)

    invoke.tasks.Task.argspec = patched_task_argspec


fix_annotations()



"""Invoke commands for all things build!"""
from invoke import Runner, task

# Name of the repository
REPO_NAME = 'core-client-py'


@task
def publish_dev_build(cmd: Runner, token_name: str, token: str) -> None:
    """Publish a development build."""
    # The first step is to build the sdist and wheel
    cmd.run('poetry build')

    # TODO: (alexn) get correct package location
    # Make sure the package registry is available
    # cmd.run(f'poetry config repositories.{REPO_NAME} https://git.mantiumai.com/api/v4/projects/153/packages/pypi')

    # Set the HTTP-Auth used when connecting to GitLab's PyPI interface
    cmd.run(f'poetry config http-basic.{REPO_NAME} "{token_name}" "{token}"')

    # Now it's time to publish to the repository package registry
    cmd.run(f'poetry publish -r {REPO_NAME}')


@task
def publish_build(cmd: Runner) -> None:
    """Publish a build. Intended to be run via the GitLab CI system."""
    # The first step is to build the sdist and wheel
    cmd.run('poetry build')

    # TODO: (alexn) configure CI variables to work with this
    # Set the repo
    cmd.run(f'poetry config repositories.{REPO_NAME} ${{CI_API_V4_URL}}/projects/${{CI_PROJECT_ID}}/packages/pypi')

    # Set the repo credentials
    cmd.run(f'poetry config http-basic.{REPO_NAME} gitlab-ci-token "${{CI_JOB_TOKEN}}" ')

    # Now it's time to publish to the repository package registry
    cmd.run(f'poetry publish -r {REPO_NAME}')



"""Monkeypatch for getting annotations to work with Invoke tasks."""
import sys
from inspect import ArgSpec, getfullargspec
from typing import Any, Callable
from unittest.mock import patch

import invoke


def fix_annotations() -> None:
    """Pyinvoke doesnt accept annotations by default, this fixes that.

    Based on: https://github.com/pyinvoke/invoke/pull/606
    """

    def patched_inspect_getargspec(func: Callable) -> ArgSpec:
        """Patch inspect.getargspec to use inspect.getfullargspec."""
        spec = getfullargspec(func)
        return ArgSpec(*spec[0:4])  # type: ignore

    org_task_argspec = invoke.tasks.Task.argspec

    def patched_task_argspec(*args: Any, **kwargs: Any) -> ArgSpec:
        """Patch invoke.tasks.Task.argspec to use patched_inspect_getargspec."""
        with patch(target='inspect.getargspec', new=patched_inspect_getargspec):
            return org_task_argspec(*args, **kwargs)

    invoke.tasks.Task.argspec = patched_task_argspec    # type: ignore


def get_base_prefix_compat() -> str:
    """Get base/real prefix, or sys.prefix if there is none."""
    return getattr(sys, 'base_prefix', None) or getattr(sys, 'real_prefix', None) or sys.prefix


def in_virtualenv() -> bool:
    """Check if you're in a virtualenv"""
    return get_base_prefix_compat() != sys.prefix
